@@ -7,9 +7,12 @@ from matplotlib import path
 import matplotlib.pyplot as plt
 from scipy import interpolate
 from shapely import geometry as geo
+from shapely import affinity
+from shapely.validation import explain_validity
 
 from ..geometry import close_curve
 from ..parameter import Parameter
+from .mesh import generate_mesh, optimize_mesh
 
 
 logger = logging.getLogger(__name__)
@@ -32,6 +35,8 @@ class Layer(object):
         z0: Vertical location of the layer.
     """
 
+    __slots__ = ("name", "thickness", "london_lambda", "z0", "_Lambda")
+
     def __init__(
         self,
         name: str,
@@ -120,6 +125,8 @@ class Polygon(object):
         mesh: Whether to include this polygon when computing a mesh.
     """
 
+    __slots__ = ("name", "layer", "_points", "mesh")
+
     def __init__(
         self,
         name: Optional[str] = None,
@@ -159,7 +166,11 @@ class Polygon(object):
         if points.interiors:
             raise ValueError("Expected a simply-connected polygon.")
         if not points.is_valid:
-            raise ValueError("The given points do not define a valid polygon.")
+            reason = explain_validity(points)
+            raise ValueError(
+                "The given points do not define a valid polygon for the following "
+                f"reason: {reason}."
+            )
         points = close_curve(np.array(points.exterior.coords))
         if points.ndim != 2 or points.shape[-1] != 2:
             raise ValueError(f"Expected shape (n, 2), but got {points.shape}.")
@@ -251,6 +262,136 @@ class Polygon(object):
             return np.where(boundary)[0]
         return boundary
 
+    def make_mesh(
+        self,
+        min_points: Optional[int] = None,
+        optimesh_steps: Optional[int] = None,
+        optimesh_method: str = "cvt-block-diagonal",
+        optimesh_tolerance: float = 1e-3,
+        optimesh_verbose: bool = False,
+        **meshpy_kwargs,
+    ) -> Tuple[np.ndarray, np.ndarray]:
+        """Returns the vertices and triangles of a Delaunay mesh covering the Polygon.
+
+        Args:
+            min_points: Minimum number of vertices in the mesh. If None, then
+                the number of vertices will be determined by meshpy_kwargs and the
+                number of vertices in the underlying polygons.
+            optimesh_steps: Maximum number of optimesh steps. If None, then no
+                optimization is done.
+            optimesh_method: Name of the optimization method to use.
+            optimesh_tolerance: Optimesh quality tolerance.
+            optimesh_verbose: Whether to use verbose mode in optimesh.
+            **meshpy_kwargs: Passed to meshpy.triangle.build().
+
+        Returns:
+            Mesh vertex coordinates and triangle indices
+        """
+        points, triangles = generate_mesh(
+            self.points,
+            min_points=min_points,
+            convex_hull=False,
+            **meshpy_kwargs,
+        )
+        if optimesh_steps:
+            logger.debug(f"Optimizing mesh with {points.shape[0]} vertices.")
+            try:
+                points, triangles = optimize_mesh(
+                    points,
+                    triangles,
+                    optimesh_steps,
+                    method=optimesh_method,
+                    tolerance=optimesh_tolerance,
+                    verbose=optimesh_verbose,
+                )
+            except np.linalg.LinAlgError as e:
+                err = (
+                    "LinAlgError encountered in optimesh. Try reducing min_points "
+                    "or increasing the number of points in the Polygon."
+                )
+                raise RuntimeError(err) from e
+        logger.debug(
+            f"Finished generating mesh with {points.shape[0]} points and "
+            f"{triangles.shape[0]} triangles."
+        )
+        return points, triangles
+
+    def rotate(
+        self,
+        degrees: float,
+        origin: Union[str, Tuple[float, float]] = (0.0, 0.0),
+        inplace: bool = False,
+    ) -> "Polygon":
+        """Rotates the polygon counterclockwise by a given angle.
+
+        Args:
+            degrees: The amount by which to rotate the polygon.
+            origin: (x, y) coorindates about which to rotate, or the strings
+                "center" (for the bounding box center) or "centroid"
+                (for the polygon center of mass).
+            inplace: If True, modify the polygon in place. Otherwise, return
+                a modified copy.
+
+        Returns:
+            The rotated polygon.
+        """
+        polygon = self if inplace else self.copy()
+        polygon.points = affinity.rotate(
+            self.polygon, degrees, origin=origin, use_radians=False
+        )
+        return polygon
+
+    def translate(
+        self,
+        dx: float = 0.0,
+        dy: float = 0.0,
+        inplace: bool = False,
+    ) -> "Polygon":
+        """Translates the polygon by a given distance.
+
+        Args:
+            dx: Distance by which to translate along the x-axis.
+            dy: Distance by which to translate along the y-axis.
+            inplace: If True, modify the polygon in place. Otherwise, return
+                a modified copy.
+
+        Returns:
+            The translated polygon.
+        """
+        polygon = self if inplace else self.copy()
+        polygon.points = affinity.translate(self.polygon, xoff=dx, yoff=dy)
+        return polygon
+
+    def scale(
+        self,
+        xfact: float = 1.0,
+        yfact: float = 1.0,
+        origin: Union[str, Tuple[float, float]] = (0, 0),
+        inplace: bool = False,
+    ) -> "Polygon":
+        """Scales the polygon horizontally by ``xfact`` and vertically by ``yfact``.
+
+        Negative ``xfact`` (``yfact``) can be used to reflect the polygon horizontally
+        (vertically) about the ``origin``.
+
+        Args:
+            xfact: Distance by which to translate along the x-axis.
+            yfact: Distance by which to translate along the y-axis.
+            origin: (x, y) coorindates for the scaling origin, or the strings
+                "center" (for the bounding box center) or "centroid"
+                (for the polygon center of mass).
+            inplace: If True, modify the polygon in place. Otherwise, return
+                a modified copy.
+
+        Returns:
+            The scaled polygon.
+        """
+        polygon = self if inplace else self.copy()
+        polygon.points = affinity.scale(
+            self.polygon, xfact=xfact, yfact=yfact, origin=origin
+        )
+        return polygon
+
     def _join_via(
         self,
         other: Union[
@@ -288,7 +429,7 @@ class Polygon(object):
                 and other.layer is not None
                 and self.layer != other.layer
             ):
-                logger.warn(
+                logger.warning(
                     f"Taking the {operation} of {self} and {other} even though "
                     f"they are assigned to different layers."
                 )
@@ -299,13 +440,17 @@ class Polygon(object):
                 f"Valid types are {(Polygon, ) + valid_types}, got {type(other)}."
             )
         joined = getattr(self.polygon, operation)(other_poly)
-        if (
-            not isinstance(joined, geo.polygon.Polygon)
-            or joined.is_empty
-            or not joined.is_valid
-        ):
+        reason = None
+        if not isinstance(joined, geo.polygon.Polygon):
+            reason = f"joined polygon has an unexpected type ({type(joined)})"
+        elif joined.is_empty:
+            reason = "joined polygon is empty"
+        elif not joined.is_valid:
+            reason = explain_validity(joined)
+        if reason is not None:
             raise ValueError(
-                f"The {operation} of the two polygons is not a valid polygon."
+                f"The {operation} of the two polygons is not a valid polygon "
+                f"for the following reason: {reason}."
             )
         return joined
 
