@@ -3,29 +3,25 @@ import json
 import logging
 from copy import deepcopy
 from collections import defaultdict
+from contextlib import contextmanager
 from typing import Optional, Union, List, Tuple, Dict, Any
 
 import dill
-import pint
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.path import Path
 from matplotlib.patches import PathPatch
 import scipy.sparse as sp
-from scipy import spatial
-from meshpy import triangle
-import optimesh
 
-from .. import solve
 from .. import fem
+from ..units import ureg
 from ..parameter import Parameter
 from .components import Layer, Polygon
+from .mesh import generate_mesh, optimize_mesh
 
 
 logger = logging.getLogger(__name__)
 
-ureg = pint.UnitRegistry()
-
 
 class Device(object):
     """An object representing a device composed of multiple layers of
@@ -342,6 +338,132 @@ class Device(object):
         # Deep copy (copy all arrays and return references to copies)
         return self.copy(with_arrays=True, copy_arrays=True)
 
+    def _warn_if_mesh_exist(self, method: str) -> None:
+        if self.points is None and self.triangles is None:
+            return
+        message = (
+            f"Calling device.{method} on a device whose mesh already exists returns "
+            f"a new device with no mesh. Call new_device.make_mesh() to generate the mesh "
+            f"for the new device."
+        )
+        logger.warning(message)
+
+    def scale(
+        self, xfact: float = 1, yfact: float = 1, origin: Tuple[float, float] = (0, 0)
+    ) -> "Device":
+        """Returns a new device with polygons scaled horizontally and/or vertically.
+
+        Negative ``xfact`` (``yfact``) can be used to reflect the device horizontally
+        (vertically) about the ``origin``.
+
+        Args:
+            xfact: Factor by which to scale the device horizontally.
+            yfact: Factor by which to scale the device vertically.
+            origin: (x, y) coorindates of the origin.
+
+        Returns:
+            The scaled :class:`superscreen.device.device.Device`.
+        """
+        if not (
+            isinstance(origin, tuple)
+            and len(origin) == 2
+            and all(isinstance(val, (int, float)) for val in origin)
+        ):
+            raise TypeError("Origin must be a tuple of floats (x, y).")
+        self._warn_if_mesh_exist("scale()")
+        device = self.copy(with_arrays=False)
+        for polygon in device.polygons.values():
+            polygon.scale(xfact=xfact, yfact=yfact, origin=origin, inplace=True)
+        return device
+
+    def rotate(self, degrees: float, origin: Tuple[float, float] = (0, 0)) -> "Device":
+        """Returns a new device with polygons rotated a given amount
+        counterclockwise about specified origin.
+
+        Args:
+            degrees: The amount by which to rotate the polygons.
+            origin: (x, y) coorindates of the origin.
+
+        Returns:
+            The rotated :class:`superscreen.device.device.Device`.
+        """
+        if not (
+            isinstance(origin, tuple)
+            and len(origin) == 2
+            and all(isinstance(val, (int, float)) for val in origin)
+        ):
+            raise TypeError("Origin must be a tuple of floats (x, y).")
+        self._warn_if_mesh_exist("rotate()")
+        device = self.copy(with_arrays=False)
+        for polygon in device.polygons.values():
+            polygon.rotate(degrees, origin=origin, inplace=True)
+        return device
+
+    def mirror_layers(self, about_z: float = 0.0) -> "Device":
+        """Returns a new device with its layers mirrored about the plane
+        ``z = about_z``.
+
+        Args:
+            about_z: The z-position of the plane (parallel to the x-y plane)
+                about which to mirror the layers.
+
+        Returns:
+            The mirrored :class:`superscreen.device.device.Device`.
+        """
+        device = self.copy(with_arrays=True, copy_arrays=True)
+        for layer in device.layers.values():
+            layer.z0 = about_z - layer.z0
+        return device
+
+    def translate(
+        self,
+        dx: float,
+        dy: float,
+        dz: float = 0,
+        inplace: bool = False,
+    ) -> "Device":
+        """Translates the device polygons, layers, and mesh in space by a given amount.
+
+        Args:
+            dx: Distance by which to translate along the x-axis.
+            dy: Distance by which to translate along the y-axis.
+            dz: Distance by which to translate layers along the z-axis.
+            inplace: If True, modifies the device (``self``) in-place and returns None,
+                otherwise, creates a new device, translates it, and returns it.
+
+        Returns:
+            The translated device.
+        """
+        if inplace:
+            device = self
+        else:
+            self._warn_if_mesh_exist("translate(..., inplace=False)")
+            device = self.copy(with_arrays=True, copy_arrays=True)
+        for polygon in device.polygons.values():
+            polygon.translate(dx, dy, inplace=True)
+        if device.points is not None:
+            device.points += np.array([[dx, dy]])
+        if dz:
+            for layer in device.layers.values():
+                layer.z0 += dz
+        return device
+
+    @contextmanager
+    def translation(self, dx: float, dy: float, dz: float = 0) -> None:
+        """A context manager that temporarily translates a device in-place,
+        then returns it to its original position.
+
+        Args:
+            dx: Distance by which to translate polygons along the x-axis.
+            dy: Distance by which to translate polygons along the y-axis.
+            dz: Distance by which to translate layers along the z-axis.
+        """
+        try:
+            self.translate(dx, dy, dz=dz, inplace=True)
+            yield
+        finally:
+            self.translate(-dx, -dy, dz=-dz, inplace=True)
+
     def make_mesh(
         self,
         compute_matrices: bool = True,
@@ -358,8 +480,8 @@ class Device(object):
         Args:
             compute_matrices: Whether to compute the field-independent matrices
                 (weights, Q, Laplace operator) needed for Brandt simulations.
-            weight_method: Meshing scheme: either "uniform", "half_cotangent",
-                or "inv_euclidian".
+            weight_method: Weight methods for computing the Laplace operator:
+                one of "uniform", "half_cotangent", or "inv_euclidian".
             min_points: Minimum number of vertices in the mesh. If None, then
                 the number of vertices will be determined by meshpy_kwargs and the
                 number of vertices in the underlying polygons.
@@ -371,50 +493,22 @@ class Device(object):
             **meshpy_kwargs: Passed to meshpy.triangle.build().
         """
         logger.info("Generating mesh...")
-        # Mesh the entire convex hull of poly_points
         poly_points = self.poly_points
-        hull = spatial.ConvexHull(poly_points)
-        mesh_info = triangle.MeshInfo()
-        mesh_info.set_points(poly_points)
-        mesh_info.set_facets(hull.simplices)
-        # Optimal angle is 60 degrees,
-        # meshpy quality meshing default is 20 degrees
-        min_angle = meshpy_kwargs.get("min_angle", 32.5)
-        meshpy_kwargs["min_angle"] = min_angle
-        if min_points is None:
-            mesh = triangle.build(mesh_info=mesh_info, **meshpy_kwargs)
-            points = np.array(mesh.points)
-            triangles = np.array(mesh.elements)
-        else:
-            max_vol = 1
-            num_points = 0
-            kwargs = meshpy_kwargs.copy()
-            i = 1
-            while num_points < min_points:
-                kwargs["max_volume"] = max_vol
-                mesh = triangle.build(
-                    mesh_info=mesh_info,
-                    **kwargs,
-                )
-                points = np.array(mesh.points)
-                triangles = np.array(mesh.elements)
-                num_points = points.shape[0]
-                logger.debug(
-                    f"Iteration {i}: Made mesh with {points.shape[0]} points and "
-                    f"{triangles.shape[0]} triangles using max_volume={max_vol:.2e}. "
-                    f"Target number of points: {min_points}."
-                )
-                max_vol *= 0.9
-                i += 1
+        points, triangles = generate_mesh(
+            poly_points,
+            min_points=min_points,
+            convex_hull=True,
+            **meshpy_kwargs,
+        )
         if optimesh_steps:
             logger.info(f"Optimizing mesh with {points.shape[0]} vertices.")
             try:
-                points, triangles = optimesh.optimize_points_cells(
+                points, triangles = optimize_mesh(
                     points,
                     triangles,
-                    optimesh_method,
-                    optimesh_tolerance,
                     optimesh_steps,
+                    method=optimesh_method,
+                    tolerance=optimesh_tolerance,
                     verbose=optimesh_verbose,
                 )
             except np.linalg.LinAlgError as e:
@@ -444,6 +538,8 @@ class Device(object):
                 or "inv_euclidian".
         """
 
+        from ..solve import C_vector, q_matrix, Q_matrix
+
         points = self.points
         triangles = self.triangles
 
@@ -454,7 +550,7 @@ class Device(object):
             )
 
         logger.info("Calculating weight matrix.")
-        self.weights = fem.mass_matrix(points, triangles, sparse=False)
+        self.weights = fem.mass_matrix(points, triangles)
 
         logger.info("Calculating Laplace operator.")
         self.Del2 = fem.laplace_operator(
@@ -466,9 +562,9 @@ class Device(object):
         )
         logger.info("Calculating kernel matrix.")
         solve_dtype = self.solve_dtype
-        q = solve.q_matrix(points, dtype=solve_dtype)
-        C = solve.C_vector(points, dtype=solve_dtype)
-        self.Q = solve.Q_matrix(q, C, self.weights, dtype=solve_dtype)
+        q = q_matrix(points, dtype=solve_dtype)
+        C = C_vector(points, dtype=solve_dtype)
+        self.Q = Q_matrix(q, C, self.weights, dtype=solve_dtype)
 
     def mutual_inductance_matrix(
         self,
