@@ -11,27 +11,27 @@ from typing import (
     List,
     Dict,
     Any,
-    TYPE_CHECKING,
 )
 
 import pint
 import psutil
 import numpy as np
 import scipy.sparse as sp
+import scipy.linalg as la
 from scipy.spatial import distance
 
 from .io import NullContextManager
 from .parameter import Constant, Parameter
 from .solution import Solution, Vortex
 from .sources import ConstantField
-
-if TYPE_CHECKING:
-    from .device import Device
+from .device import Device
 
 
 lambda_str = "\u03bb"
 Lambda_str = "\u039b"
 
+logger = logging.getLogger(__name__)
+
 
 def q_matrix(
     points: np.ndarray, dtype: Optional[Union[str, np.dtype]] = None
@@ -233,7 +233,7 @@ def field_conversion_factor(
 
 def solve_layer(
     *,
-    device: "Device",
+    device: Device,
     layer: str,
     applied_field: Union[Callable, np.ndarray],
     circulating_currents: Optional[Dict[str, Union[float, str, pint.Quantity]]] = None,
@@ -267,7 +267,6 @@ def solve_layer(
     Returns:
         stream function, total field, film screening field
     """
-    logger = logging.getLogger(__name__)
 
     circulating_currents = circulating_currents or {}
     for name in circulating_currents:
@@ -283,9 +282,6 @@ def solve_layer(
         .magnitude
     )
 
-    if device.weights is None:
-        device.make_mesh(compute_matrices=True)
-
     if weights is None:
         weights = device.weights
     if Del2 is None:
@@ -366,13 +362,9 @@ def solve_layer(
         # Eqs. 15-17 in [Brandt], Eqs 12-14 in [Kirtley1], Eqs. 12-14 in [Kirtley2].
         A = Q[ix2d] * weights[ix1d] - Lambda[ix1d] * Del2[ix2d]
         h = Hz_applied[ix1d] - Ha_eff[ix1d]
-        # In some simple benchmarking on an Intel Mac with numpy=1.20.3 and scipy=1.6.3
-        # I have found numpy.linalg.inv to be a bit faster than scipy.linalg.inv,
-        # but I may want to revisit this in the future.
-        # See also: https://docs.scipy.org/doc/scipy/reference/tutorial/
-        # linalg.html#scipy-linalg-vs-numpy-linalg
-        K = np.linalg.inv(A)
-        gf = -K @ h
+        lu, piv = la.lu_factor(-A)
+        gf = la.lu_solve((lu, piv), h)
+
         g[ix1d] = gf
         if check_inversion:
             # Validate solution
@@ -382,10 +374,13 @@ def solve_layer(
                     f"Unable to solve for stream function in {layer} ({name}), "
                     f"maximum error {np.abs(hsim - h).max():.3e}."
                 )
-
+        K = None  # Matrix inverse of A
         for vortex in film_to_vortices[name]:
+            if K is None:
+                # Compute K only once if needed
+                K = -la.lu_solve((lu, piv), np.eye(A.shape[0]))
             # Index of the mesh vertex that is closest to the vortex position:
-            # in the film-specific subarray
+            # in the film-specific sub-mesh
             j_film = np.argmin(
                 np.sum(np.square(points[ix1d] - [[vortex.x, vortex.y]]), axis=1)
             )
@@ -403,7 +398,7 @@ def solve_layer(
 
 
 def solve(
-    device: "Device",
+    device: Device,
     *,
     applied_field: Optional[Callable] = None,
     circulating_currents: Optional[Dict[str, Union[float, str, pint.Quantity]]] = None,
@@ -466,13 +461,20 @@ def solve(
     if log_level is not None:
         logging.basicConfig(level=log_level)
 
-    logger = logging.getLogger(__name__)
-
     if directory is not None:
         os.makedirs(directory, exist_ok=True)
 
-    dtype = device.solve_dtype
+    if device.points is None:
+        raise ValueError(
+            "The device does not have a mesh. Call device.make_mesh() to generate it."
+        )
+    if device.weights is None or device.Del2 is None:
+        raise ValueError(
+            "The device does not have a Laplace operator. "
+            "Call device.compute_matrices() to calculate it."
+        )
 
+    dtype = device.solve_dtype
     points = device.points.astype(dtype, copy=False)
     weights = device.weights.astype(dtype, copy=False)
     Del2 = device.Del2
@@ -737,12 +739,14 @@ def solve(
                     solutions.append(solution)
                 else:
                     del solution
+        if cache_kernels_to_disk:
+            context.cleanup()
     if return_solutions:
         return solutions
 
 
 def solve_many(
-    device: "Device",
+    device: Device,
     *,
     parallel_method: Optional[str] = None,
     applied_fields: Optional[Union[Parameter, List[Parameter]]] = None,
@@ -766,6 +770,7 @@ def solve_many(
     cache_memory_cutoff: float = np.inf,
     log_level: int = logging.INFO,
     use_shared_memory: bool = True,
+    num_cpus: Optional[int] = None,
 ) -> Tuple[Optional[Union[List[Solution], List[List[Solution]]]], Optional[List[str]]]:
     """Solves many models involving the same device, optionally in parallel using
     multiple processes.
@@ -809,6 +814,7 @@ def solve_many(
             by the operating system.
         log_level: Logging level to use, if any.
         use_shared_memory: Whether to use shared memory if parallel_method is not None.
+        num_cpus: The number of processes to utilize.
 
     Returns:
         solutions, paths. If return_solutions is True, solutions is either a list of
@@ -832,6 +838,10 @@ def solve_many(
             f"Unknown parallel method, {parallel_method}. "
             f"Valid methods are {list(parallel_methods)}."
         )
+    if num_cpus is not None and parallel_method in (None, False, "serial"):
+        logger.warning(
+            f"Ignoring num_cpus because parallel_method = {parallel_method!r}."
+        )
 
     solutions, paths = parallel_methods[parallel_method](
         device=device,
@@ -851,6 +861,7 @@ def solve_many(
         cache_memory_cutoff=cache_memory_cutoff,
         log_level=log_level,
         use_shared_memory=use_shared_memory,
+        num_cpus=num_cpus,
     )
 
     return solutions, paths
@@ -878,5 +889,5 @@ def _patch_docstring(func):
 from .parallel import solve_many_serial, solve_many_mp, solve_many_ray  # noqa: E402
 
 
-for func in [solve_many_serial, solve_many_mp, solve_many_ray]:
+for func in (solve_many_serial, solve_many_mp, solve_many_ray):
     _patch_docstring(func)
